import uuid

from astrbot.api import logger
from astrbot.core.star.context import Context

from .booters.base import SandboxBooter

session_booter: dict[str, SandboxBooter] = {}


async def get_booter(
    context: Context,
    session_id: str,
) -> SandboxBooter:
    config = context.get_config(umo=session_id)

    sandbox_cfg = config.get("provider_settings", {}).get("sandbox", {})
    booter_type = sandbox_cfg.get("booter", "shipyard")

    if session_id in session_booter:
        booter = session_booter[session_id]
        if not await booter.available():
            # rebuild
            session_booter.pop(session_id, None)
    if session_id not in session_booter:
        uuid_str = uuid.uuid5(uuid.NAMESPACE_DNS, session_id).hex
        if booter_type == "shipyard":
            from .booters.shipyard import ShipyardBooter

            ep = sandbox_cfg.get("shipyard_endpoint", "")
            token = sandbox_cfg.get("shipyard_access_token", "")
            ttl = sandbox_cfg.get("shipyard_ttl", 3600)
            max_sessions = sandbox_cfg.get("shipyard_max_sessions", 10)

            client = ShipyardBooter(
                endpoint_url=ep, access_token=token, ttl=ttl, session_num=max_sessions
            )
        elif booter_type == "boxlite":
            from .booters.boxlite import BoxliteBooter

            client = BoxliteBooter()
        else:
            raise ValueError(f"Unknown booter type: {booter_type}")

        try:
            await client.boot(uuid_str)
        except Exception as e:
            logger.error(f"Error booting sandbox for session {session_id}: {e}")
            raise e

        session_booter[session_id] = client
    return session_booter[session_id]



from dataclasses import dataclass, field

import mcp

from astrbot.api import FunctionTool
from astrbot.core.agent.run_context import ContextWrapper
from astrbot.core.agent.tool import ToolExecResult
from astrbot.core.astr_agent_context import AstrAgentContext
from astrbot.core.sandbox.sandbox_client import get_booter


@dataclass
class PythonTool(FunctionTool):
    name: str = "astrbot_execute_ipython"
    description: str = "Execute a command in an IPython shell."
    parameters: dict = field(
        default_factory=lambda: {
            "type": "object",
            "properties": {
                "code": {
                    "type": "string",
                    "description": "The Python code to execute.",
                },
                "silent": {
                    "type": "boolean",
                    "description": "Whether to suppress the output of the code execution.",
                    "default": False,
                },
            },
            "required": ["code"],
        }
    )

    async def call(
        self, context: ContextWrapper[AstrAgentContext], code: str, silent: bool = False
    ) -> ToolExecResult:
        sb = await get_booter(
            context.context.context,
            context.context.event.unified_msg_origin,
        )
        try:
            result = await sb.python.exec(code, silent=silent)
            data = result.get("data", {})
            output = data.get("output", {})
            error = data.get("error", "")
            images: list[dict] = output.get("images", [])
            text: str = output.get("text", "")

            resp = mcp.types.CallToolResult(content=[])

            if error:
                resp.content.append(
                    mcp.types.TextContent(type="text", text=f"error: {error}")
                )

            if images:
                for img in images:
                    resp.content.append(
                        mcp.types.ImageContent(
                            type="image", data=img["image/png"], mimeType="image/png"
                        )
                    )
            if text:
                resp.content.append(mcp.types.TextContent(type="text", text=text))

            if not resp.content:
                resp.content.append(
                    mcp.types.TextContent(type="text", text="No output.")
                )

            return resp

        except Exception as e:
            return f"Error executing code: {str(e)}"
